package eksempel.rest;

import eksempel.helper.FileHelper;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

public abstract class AbstractIndlaesActionListener<T extends RegisterInput> implements ActionListener {
    protected String errorText;
    protected JFrame parent;
    protected T input;

    public void actionPerformed(ActionEvent ignore) {
        if (isAllOk()) {
            errorText = "";

            // Hent indhold
            String indhold = "";
            try {
                indhold = FileHelper.readTextFile(getFilename());
            } catch (IOException e) {
                errorText = kanIkkeLaeseDokumentTekst() + e.getMessage();
            }

            String supplerendeInstans = null;
            if (getSupplerendeInstansFilnavn() != null && !getSupplerendeInstansFilnavn().trim().equals("")) {
                try {
                    supplerendeInstans = FileHelper.readTextFile(getSupplerendeInstansFilnavn().trim());
                } catch (IOException e1) {
                    errorText = "Kan ikke læse supplerende instans XBRL dokument: \n" + e1.getMessage();
                }
            }

            input = samlInput(indhold, supplerendeInstans);

            // Test for fejl
            if (errorText.equals("")) {
                // Kald REST endpoint
                String errorMsg = null;
                String result = null;
                try {
                    parent.setCursor(new Cursor(Cursor.WAIT_CURSOR));
                    result = invokeRegister(input);
                    parent.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                } catch (Exception e1) {
                    e1.printStackTrace();
                    errorMsg = e1.getMessage();
                    parent.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                }

                // Vis resultatet i dialog
                if (errorMsg != null) {
                    JTextArea text = new JTextArea(errorMsg, 0, 100);
                    text.setLineWrap(true);
                    text.setSize(new Dimension(480, 10));
                    text.setPreferredSize(new Dimension(480, text.getPreferredSize().height));
                    JOptionPane.showMessageDialog(parent, text, "Fejl ved kald af REST endpoint", JOptionPane.ERROR_MESSAGE);
                } else {
                    JTextArea text = new JTextArea(result, 0, 75);
                    text.setLineWrap(true);
                    text.setSize(new Dimension(480, 10));
                    text.setPreferredSize(new Dimension(480, text.getPreferredSize().height));
                    JOptionPane.showMessageDialog(parent, text, "Resultat af kald til REST endpoint", JOptionPane.INFORMATION_MESSAGE);
                }
            } else {
                JOptionPane.showMessageDialog(parent, errorText, "Fejl ved behandling af data:", JOptionPane.ERROR_MESSAGE);
            }
        } else {
            errorText = derSkalVaelgesDokumentSamtAfsenderCertifikatOgPasswordTekst();
            JOptionPane.showMessageDialog(parent, errorText, "Der mangler obligatoriske felter", JOptionPane.ERROR_MESSAGE);
        }
    }

    protected abstract String derSkalVaelgesDokumentSamtAfsenderCertifikatOgPasswordTekst();

    protected abstract String kanIkkeLaeseDokumentTekst();

    protected abstract String invokeRegister(T input) throws Exception;

    protected abstract String getSupplerendeInstansFilnavn();

    protected abstract T samlInput(String indhold, String supplerendeInstans);

    /**
     * Valider input: Inline XBRL samt FOCES certifikat skal være udfyldt
     */
    protected abstract boolean isAllOk();

    protected abstract String getFilename();
}
