package eksempel.rest;

import eksempel.rest.tls.InsecureHostnameVerifier;
import eksempel.rest.tls.InsecureTrustManager;
import org.apache.http.ssl.SSLContexts;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.Invocation;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.Base64;

abstract public class AbstractRegisterDocumentInvoker<I extends RegisterInput<B>, B extends RegisterBody, T extends RegisterResponse> {

    private final Class<T> clazz;

    public AbstractRegisterDocumentInvoker(Class<T> clazz) {
        this.clazz = clazz;
    }

    abstract protected String getController();

    abstract protected String getAction();

    protected String urlFor(boolean useProd, boolean usePreprod, boolean useTest, boolean useDev) {
        String urlToUse;
        if (useProd) {
            urlToUse = "://erst-api.virk.dk";
        } else if (usePreprod) {
            urlToUse = "://erst-apipreprod.virk.dk";
        } else if (useTest) {
            urlToUse = "://erst-apitest.virk.dk";
        } else if (useDev) {
            urlToUse = "://erst-apidev.virk.dk";
        } else {
            urlToUse = "://localhost.erstdev.dk";
        }
        return urlToUse;
    }

    /**
     * Kalder REST endpoint, med det valgte XBRL dokument
     *
     * @param input En {@link RegisterInput} der indeholder XBRL, InlineXBRL og/eller PDF dokumentet samt tilhørende data
     * @param useHttps Angivelse af om kaldet til REST services skal gå via https eller http
     *
     * @param openDocs
     * @return returværdi fra REST endpoint'et
     */
    public String register(I input, boolean useHttps, boolean useProd, boolean usePreprod, boolean useTest, boolean useDev, boolean useLocal, boolean openDocs, String focesSendFile, String focesSendPw) throws Exception {
        long starttidspunkt = System.currentTimeMillis();
        T response = postToServer(useHttps, urlFor(useProd, usePreprod, useTest, useDev), useLocal, focesSendFile, focesSendPw, input.createBody(newBody()), getController(), getAction());
        long duration = System.currentTimeMillis() - starttidspunkt;

        System.out.println(response);

        String result = "REST service kaldet tog " + duration + " ms.\n" +
                response.responseInformation();

        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        StringSelection stringSelection = new StringSelection(response.getUniqueReportIdentifier());
        clipboard.setContents(stringSelection, null);

        efterbraender(openDocs, response);

        return result;
    }

    protected abstract B newBody();

    protected T postToServer(boolean useHttps, String urlToUse, boolean useLocal, String focesSendFile, String focesSendPw, B body, String controller, String action)
            throws NoSuchAlgorithmException, KeyManagementException, UnrecoverableKeyException, CertificateException, KeyStoreException, IOException {
        String path = useLocal ? "regnskab-indberetning-web/api/" + controller + "/" + action : "ri/api/" + controller + "/" + action;

        Response response = this.executeClientPost(body, useHttps, useLocal, focesSendFile, focesSendPw, urlToUse, path);

        if (response.getStatus() != 200) {
            throw new IOException("Kaldet gik ikke godt! Serveren returnerede status " + response.getStatus());
        }
        return response.readEntity(clazz);
    }

    private Response executeClientPost(RegisterBody body, boolean useHttps, boolean useLocal, String focesSendFile, String focesSendPw, String baseUrl, String path)
            throws NoSuchAlgorithmException, KeyManagementException, KeyStoreException, IOException, CertificateException, UnrecoverableKeyException {

        String protocol = useHttps ? "https" : "http";

        System.out.println("Benytter endpoint: " + protocol + baseUrl + "/" + path);

        KeyStore keyStore = KeyStore.getInstance("PKCS12");
        keyStore.load(new FileInputStream(focesSendFile), focesSendPw.toCharArray());

        ClientBuilder clientBuilder;
        if (useHttps) {
            if (useLocal) {
                SSLContext sslContext = SSLContext.getInstance("TLSv1.2");
                System.setProperty("https.protocols", "TLSv1.2");

                TrustManager[] trustAllCerts = {new InsecureTrustManager()};
                sslContext.init(null, trustAllCerts, new java.security.SecureRandom());
                HostnameVerifier allHostsValid = new InsecureHostnameVerifier();
                clientBuilder = ClientBuilder.newBuilder().sslContext(sslContext).hostnameVerifier(allHostsValid);
            } else {
                SSLContext sslContext = SSLContexts.custom()
                        .loadKeyMaterial(keyStore, focesSendPw.toCharArray())
                        .build();

                clientBuilder = ClientBuilder.newBuilder().sslContext(sslContext);
            }
        } else {
            clientBuilder = ClientBuilder.newBuilder();
        }

        Invocation.Builder builder = clientBuilder.build().target(protocol + baseUrl)
                .path(path)
                .request(MediaType.APPLICATION_JSON)
                .header("Content-Type", "application/json;charset=UTF-8");

        if (useLocal) {
            String alias = keyStore.aliases().nextElement();
            String clientCert = "-----BEGIN CERTIFICATE-----" +
                    Base64.getEncoder().encodeToString(keyStore.getCertificate(alias).getEncoded()) +
                    "-----END CERTIFICATE-----";
            clientCert = clientCert.replace("\n", "").replace("\r", "");

            return builder
                    .header("ClientCert", clientCert)
                    .post(Entity.entity(body, MediaType.APPLICATION_JSON));
        } else {
            return builder
                    .post(Entity.entity(body, MediaType.APPLICATION_JSON));
        }
    }

    public void efterbraender(boolean openDocs, T t) {}
}
