package eksempel.rest;

import eksempel.helper.FileHelper;
import eksempel.rest.xbrl.RegisterXBRLInput;
import eksempel.rest.xbrl.RegisterXBRLInvoker;

import javax.swing.*;
import java.io.IOException;

// ActionListener for indlæs knap - check (minimalt) input og initier pakning, base64 konvertering, og upload af XBRL data
class IndlaesXBRLActionListener extends AbstractIndlaesActionListener<RegisterXBRLInput> {
    protected final RegisterInlineXBRLFront gui;

    public IndlaesXBRLActionListener(JFrame parent, RegisterInlineXBRLFront gui) {
        super();
        this.parent = parent;
        this.gui = gui;
    }

    @Override
    protected String kanIkkeLaeseDokumentTekst() {
        return "Kan ikke læse XBRL dokument: \n";
    }

    @Override
    protected String derSkalVaelgesDokumentSamtAfsenderCertifikatOgPasswordTekst() {
        return "Der SKAL vælges\n\tXBRL og PDF dokument\n\tsamt afsender certifikat og password";
    }

    @Override
    protected String invokeRegister(RegisterXBRLInput input) throws Exception {
        return new RegisterXBRLInvoker().register(input, gui.httpsBox.isSelected(), gui.prodRadio.isSelected(), gui.preprodRadio.isSelected(),
                gui.testRadio.isSelected(), gui.devRadio.isSelected(), gui.localRadio.isSelected(), false,
                gui.focesSendFile.getText(), gui.focesSendPw.getText());
    }

    @Override
    protected RegisterXBRLInput samlInput(String indhold, String supplerendeInstans) {
        String pdf = null;
        // Hent PDF indhold
        try {
            pdf = FileHelper.readTextFile(gui.pdfFile.getText().trim());
        } catch (IOException e) {
            errorText = "Kan ikke læse pdf dokument: \n" + e.getMessage();
        }

        // Saml input
        RegisterXBRLInput input = new RegisterXBRLInput();
        input.setXbrl(indhold);
        input.setSupplerendeInstans(supplerendeInstans);
        input.setPdf(pdf);
        input.setToken(gui.token.getText());
        return input;
    }

    @Override
    protected String getFilename() {
        return gui.xbrlFile.getText();
    }

    @Override
    protected String getSupplerendeInstansFilnavn() {
        return gui.supplerendeInstansXbrlFile.getText();
    }

    /**
     * Valider input: XBRL, PDF samt FOCES certifikat skal være udfyldt
     */
    @Override
    protected boolean isAllOk() {
        return gui.xbrlFile.getText() != null && !gui.xbrlFile.getText().equals("")
                && gui.pdfFile.getText() != null && !gui.pdfFile.getText().equals("")
                && gui.focesSendFile.getText() != null && !gui.focesSendFile.getText().equals("")
                && gui.focesSendPw.getText() != null && !gui.focesSendPw.getText().equals("");
    }
}
