package eksempel.rest;

import eksempel.helper.ConstraintsBuilder;
import eksempel.listener.BrowseAction;
import eksempel.rest.ixbrl.RegisterInlineXBRLInvoker;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Opretter et vindue til at tage imod input til indlæsning af inline XBRL dokumenter og
 * benytter derefter {@link RegisterInlineXBRLInvoker} til at foretage kaldet til webservicen.
 */
public class RegisterInlineXBRLFront {

    public static void main(String[] args) {
        RegisterInlineXBRLFront registerInlineXBRLFront = new RegisterInlineXBRLFront();
        SwingUtilities.invokeLater(registerInlineXBRLFront::createAndShowGUI);
    }

    final JTextField xbrlFile = new JTextField(35);
    final JTextField inlineXbrlFile = new JTextField(35);
    final JTextField pdfFile = new JTextField(35);
    final JTextField supplerendeInstansXbrlFile = new JTextField(35);
    final JTextField ixbrlDirigent = new JTextField(35);
    final JTextField ixbrlGodkendelsesdato = new JTextField(35);
    final JTextField focesSendFile = new JTextField(35);
    final JPasswordField focesSendPw = new JPasswordField(15);
    final JTextField token = new JTextField(35);
    final JTextField testadgangstegn = new JTextField(35);
    final JRadioButton localRadio = new JRadioButton("localhost (til udviklingsbrug)", false);
    final JRadioButton devRadio = new JRadioButton("archdev (ERST Dev)", true);
    final JRadioButton testRadio = new JRadioButton("archtest (ERST Test)", false);
    final JRadioButton preprodRadio = new JRadioButton("archpreprod (ERST Preprod)", true);
    final JRadioButton prodRadio = new JRadioButton("archprod (ERST Produktion!)", true);
    final JCheckBox httpsBox = new JCheckBox("HTTPS", true);
    final JCheckBox returnerDokumenterBox = new JCheckBox("Ja", true);
    final JCheckBox aabenDokumenterBox = new JCheckBox("Ja", true);

    /**
     * Create the GUI and show it. For thread safety, this method should be invoked from the event-dispatching thread.
     */
    private void createAndShowGUI() {

        // Hovedramme med titel
        JFrame frame = new JFrame("InlineXBRL eksempel klient");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Panel til input felter/boxe
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());

        int gridy = 0;

        // Overskrift
        panel.add(new JLabel("Dialog til eksempel på implementering af Erhvervsstyrelsens system-system (REST) løsning"), new ConstraintsBuilder().gridwidth(7).gridx(0).gridy(gridy).build());

        panel.add(new JLabel(" "), new ConstraintsBuilder().gridwidth(7).gridx(0).gridy(++gridy).build());

        // Komponenter til Inline XBRL.
        JLabel inlineXbrlLabel = new JLabel("Inline XBRL dokument");
        JButton inlineXbrlBrowse = new JButton("...");
        inlineXbrlBrowse.addActionListener(new BrowseAction(frame, inlineXbrlFile));

        // Inline XBRL dokument vælger
        panel.add(inlineXbrlLabel, new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(inlineXbrlFile, new ConstraintsBuilder().gridwidth(5).gridx(1).gridy(gridy).build());
        panel.add(inlineXbrlBrowse, new ConstraintsBuilder().weightx(0.5).gridx(6).gridy(gridy).build());

        // XBRL dokument vælger
        // Komponenter til XBRL.
        JLabel xbrlLabel = new JLabel("XBRL dokument");
        xbrlLabel.setEnabled(false); // grået ud indledningsvis
        xbrlFile.setEnabled(false); // grået ud indledningsvis
        JButton xbrlBrowse = new JButton("...");
        xbrlBrowse.setEnabled(false); // grået ud indledningsvis
        xbrlBrowse.addActionListener(new BrowseAction(frame, xbrlFile));
        panel.add(xbrlLabel, new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(xbrlFile, new ConstraintsBuilder().gridwidth(5).gridx(1).gridy(gridy).build());
        panel.add(xbrlBrowse, new ConstraintsBuilder().weightx(0.5).gridx(6).gridy(gridy).build());

        // PDF dokument vælger
        // Komponenter til PDF.
        JLabel pdfLabel = new JLabel("PDF dokument");
        pdfLabel.setEnabled(false); // grået ud indledningsvis
        pdfFile.setEnabled(false); // grået ud indledningsvis
        JButton pdfBrowse = new JButton("...");
        pdfBrowse.setEnabled(false); // grået ud indledningsvis
        pdfBrowse.addActionListener(new BrowseAction(frame, pdfFile));
        panel.add(pdfLabel, new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(pdfFile, new ConstraintsBuilder().gridwidth(5).gridx(1).gridy(gridy).build());
        panel.add(pdfBrowse, new ConstraintsBuilder().weightx(0.5).gridx(6).gridy(gridy).build());

        // Supplerende instans XBRL dokument vælger
        JLabel supplerendeInstansXbrlLabel = new JLabel("Supplerende instans");
        panel.add(supplerendeInstansXbrlLabel, new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(supplerendeInstansXbrlFile, new ConstraintsBuilder().gridwidth(5).gridx(1).gridy(gridy).build());

        JButton supplerendeInstansXbrlBrowse = new JButton("...");
        supplerendeInstansXbrlBrowse.addActionListener(new BrowseAction(frame, supplerendeInstansXbrlFile));
        panel.add(supplerendeInstansXbrlBrowse, new ConstraintsBuilder().weightx(0.5).gridx(6).gridy(gridy).build());

        // Dirigent
        JLabel dirigentLabel = new JLabel("Dirigent");
        panel.add(dirigentLabel, new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(ixbrlDirigent, new ConstraintsBuilder().gridwidth(5).gridx(1).gridy(gridy).build());

        // Godkendelsesdato
        JLabel godkendelsesdatoLabel = new JLabel("Godkendelsesdato (dd-MM-yyyy)");
        panel.add(godkendelsesdatoLabel, new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(ixbrlGodkendelsesdato, new ConstraintsBuilder().gridwidth(5).gridx(1).gridy(gridy).build());

        focesSendFile.setText(System.getProperty("user.home") + "/testfiler/FOCES_gyldig.p12");
        JButton focesSendBrowse = new JButton("...");
        focesSendBrowse.addActionListener(new BrowseAction(frame, focesSendFile));

        // Inline XBRL Sender certifikat
        panel.add(new JLabel("Afsender PKCS#12 certifikat fil"), new ConstraintsBuilder().gridx(0).gridy(++gridy).build());

        panel.add(focesSendFile, new ConstraintsBuilder().gridwidth(5).gridx(1).gridy(gridy).build());

        panel.add(focesSendBrowse, new ConstraintsBuilder().weightx(0.5).gridx(6).gridy(gridy).build());

        focesSendPw.setText("Test1234");
        panel.add(new JLabel("Afsender certifikat password"), new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(focesSendPw, new ConstraintsBuilder().gridwidth(2).gridx(1).gridy(gridy).build());

        // Token
        panel.add(new JLabel("Token"), new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(token, new ConstraintsBuilder().gridwidth(2).gridx(1).gridy(gridy).build());

        panel.add(new JLabel("Adgangstegn til test"), new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(testadgangstegn, new ConstraintsBuilder().gridwidth(2).gridx(1).gridy(gridy).build());

        // XBRL / Inline XBRL vælger
        panel.add(new JLabel(" "), new ConstraintsBuilder().gridwidth(7).gridx(0).gridy(++gridy).build());

        panel.add(new JLabel("XBRL / Inline XBRL"), new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        final JRadioButton ixbrlDoktype = new JRadioButton("Inline XBRL", true);
        panel.add(ixbrlDoktype, new ConstraintsBuilder().gridx(1).gridy(gridy++).build());
        final JRadioButton xbrlDoktype = new JRadioButton("XBRL", false);
        panel.add(xbrlDoktype, new ConstraintsBuilder().gridx(1).gridy(gridy++).build());
        ButtonGroup doktypeGroup = new ButtonGroup();
        doktypeGroup.add(ixbrlDoktype);
        doktypeGroup.add(xbrlDoktype);

        // service vælger (miljø)
        panel.add(new JLabel(" "), new ConstraintsBuilder().gridwidth(7).gridx(0).gridy(++gridy).build());

        panel.add(new JLabel("Service der rammes"), new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(localRadio, new ConstraintsBuilder().gridx(1).gridy(gridy++).build());
        panel.add(devRadio, new ConstraintsBuilder().gridx(1).gridy(gridy++).build());
        panel.add(testRadio, new ConstraintsBuilder().gridx(1).gridy(gridy++).build());
        panel.add(preprodRadio, new ConstraintsBuilder().gridx(1).gridy(gridy++).build());
        panel.add(prodRadio, new ConstraintsBuilder().gridx(1).gridy(gridy++).build());

        ButtonGroup group = new ButtonGroup();
        group.add(localRadio);
        group.add(devRadio);
        group.add(testRadio);
        group.add(preprodRadio);
        group.add(prodRadio);

        // Angiv om transport via https
        panel.add(new JLabel(" "), new ConstraintsBuilder().gridwidth(7).gridx(0).gridy(gridy).build());

        panel.add(new JLabel("Transport"), new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(httpsBox, new ConstraintsBuilder().gridx(1).gridy(gridy).build());

        panel.add(new JLabel("Returner dokumenter"), new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(returnerDokumenterBox, new ConstraintsBuilder().gridx(1).gridy(gridy).build());

        panel.add(new JLabel("Åbn dokumenter"), new ConstraintsBuilder().gridx(0).gridy(++gridy).build());
        panel.add(aabenDokumenterBox, new ConstraintsBuilder().gridx(1).gridy(gridy).build());

        JButton indlaesInlineXbrl = new JButton("Indlæs inline XBRL");
        panel.add(indlaesInlineXbrl, new ConstraintsBuilder().gridx(1).gridy(++gridy).fill(GridBagConstraints.NONE).build());

        JButton testInlineXbrl = new JButton("Test inline XBRL");
        panel.add(testInlineXbrl, new ConstraintsBuilder().gridx(1).gridy(++gridy).fill(GridBagConstraints.NONE).build());

        JButton ryd = new JButton("Ryd indtastning");
        panel.add(ryd, new ConstraintsBuilder().gridx(1).gridy(++gridy).fill(GridBagConstraints.NONE).build());

        IndlaesInlineXBRLActionListener indlaesInlineXBRLActionListener = new IndlaesInlineXBRLActionListener(frame, this);
        IndlaesXBRLActionListener indlaesXBRLActionListener = new IndlaesXBRLActionListener(frame, this);
        indlaesInlineXbrl.addActionListener(indlaesInlineXBRLActionListener);

        TestInlineXBRLActionListener testInlineXBRLActionListener = new TestInlineXBRLActionListener(frame, this);
        TestXBRLActionListener testXBRLActionListener = new TestXBRLActionListener(frame, this);
        testInlineXbrl.addActionListener(testInlineXBRLActionListener);

        class DoktypeAction implements ActionListener {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (e.getActionCommand().equals("Inline XBRL")) {
                    xbrlLabel.setEnabled(false);
                    xbrlFile.setEnabled(false);
                    xbrlBrowse.setEnabled(false);
                    pdfLabel.setEnabled(false);
                    pdfFile.setEnabled(false);
                    pdfBrowse.setEnabled(false);
                    inlineXbrlLabel.setEnabled(true);
                    inlineXbrlFile.setEnabled(true);
                    inlineXbrlBrowse.setEnabled(true);
                    dirigentLabel.setEnabled(true);
                    ixbrlDirigent.setEnabled(true);
                    godkendelsesdatoLabel.setEnabled(true);
                    ixbrlGodkendelsesdato.setEnabled(true);
                    indlaesInlineXbrl.setText("Indlæs inline XBRL");
                    indlaesInlineXbrl.removeActionListener(indlaesXBRLActionListener);
                    indlaesInlineXbrl.addActionListener(indlaesInlineXBRLActionListener);
                    testInlineXbrl.setText("Test inline XBRL");
                    testInlineXbrl.removeActionListener(testXBRLActionListener);
                    testInlineXbrl.addActionListener(testInlineXBRLActionListener);
                } else if (e.getActionCommand().equals("XBRL")) {
                    xbrlLabel.setEnabled(true);
                    xbrlFile.setEnabled(true);
                    xbrlBrowse.setEnabled(true);
                    pdfLabel.setEnabled(true);
                    pdfFile.setEnabled(true);
                    pdfBrowse.setEnabled(true);
                    inlineXbrlLabel.setEnabled(false);
                    inlineXbrlFile.setEnabled(false);
                    inlineXbrlBrowse.setEnabled(false);
                    dirigentLabel.setEnabled(false);
                    ixbrlDirigent.setEnabled(false);
                    godkendelsesdatoLabel.setEnabled(false);
                    ixbrlGodkendelsesdato.setEnabled(false);
                    indlaesInlineXbrl.setText("Indlæs XBRL");
                    indlaesInlineXbrl.removeActionListener(indlaesInlineXBRLActionListener);
                    indlaesInlineXbrl.addActionListener(indlaesXBRLActionListener);
                    testInlineXbrl.setText("Test XBRL");
                    testInlineXbrl.removeActionListener(testInlineXBRLActionListener);
                    testInlineXbrl.addActionListener(testXBRLActionListener);
                }
            }
        }

        DoktypeAction documentTypeAction = new DoktypeAction();
        ixbrlDoktype.addActionListener(documentTypeAction);
        xbrlDoktype.addActionListener(documentTypeAction);

        ryd.addActionListener(e -> {
            // Nulstil input
            inlineXbrlFile.setText("");
            xbrlFile.setText("");
            pdfFile.setText("");
            supplerendeInstansXbrlFile.setText("");
            ixbrlDirigent.setText("");
            ixbrlGodkendelsesdato.setText("");
            testRadio.setSelected(true);
        });

        // Vis vinduet.
        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(panel, BorderLayout.NORTH);
        frame.pack();
        frame.setLocationRelativeTo(null);
        frame.setVisible(true);
    }
}
