package eksempel.rest.ixbrl;

import eksempel.rest.AbstractRegisterDocumentInvoker;

import java.awt.*;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Base64;

/*
 * Anvendes til at kalde REST webservices.
 */
public class RegisterInlineXBRLInvoker extends AbstractRegisterDocumentInvoker<RegisterInlineXBRLInput,RegisterInlineXBRLBody,RegisterInlineXBRLResponse> {

    public RegisterInlineXBRLInvoker() {
        super(RegisterInlineXBRLResponse.class);
    }

    @Override
    protected RegisterInlineXBRLBody newBody() {
        return new RegisterInlineXBRLBody();
    }

    public void efterbraender(boolean openDocs, RegisterInlineXBRLResponse response) {
        if (openDocs) {
            if (response.xbrl != null) {
                openMediaFile(response.xbrl, "XBRL", ".xml");
            }

            if (response.pdf != null) {
                openMediaFile(response.pdf, "PDF", ".pdf");
            }
        }
    }

    protected String getController() {
        return "ixbrl";
    }

    protected String getAction() {
        return "indberet";
    }

    private void openMediaFile(String base64EncodedContent, String type, String extension) {
        try {
            File tmpMediaFile = File.createTempFile(System.currentTimeMillis() + type, extension);
            BufferedOutputStream bw = new BufferedOutputStream(new FileOutputStream(tmpMediaFile));
            byte[] decodedBytes = Base64.getDecoder().decode(base64EncodedContent);
            bw.write(decodedBytes);
            bw.close();

            System.out.println("Midlertidig fil: type=" + type + ", extension=" + extension + ", filnavn=" + tmpMediaFile.getAbsolutePath());

            Desktop.getDesktop().browse(tmpMediaFile.toURI());
        } catch (IOException e) {
            System.err.println("Kunne ikke oprette en midlertidig " + type + " fil");
        }
    }
}
